@@ -23,37 +23,46 @@ func TestParse(t *testing.T) {
 	}
 	wantAt := ConnectionParams{
 		CommonParams: CommonParams{
-			Username:      "cc",
-			Password:      NewPassword("c@c*1"),
-			ConnectString: "192.168.1.1/cc",
-		},
-		PoolParams: PoolParams{
-			MaxLifeTime:    DefaultMaxLifeTime,
-			SessionTimeout: DefaultSessionTimeout,
-			WaitTimeout:    DefaultWaitTimeout,
-		},
-	}
-	wantDefault := ConnectionParams{
-		StandaloneConnection: DefaultStandaloneConnection,
-		CommonParams: CommonParams{
-			Username:      "user",
-			Password:      NewPassword("pass"),
-			ConnectString: "sid",
-		},
-		ConnParams: ConnParams{
-			ConnClass: DefaultConnectionClass,
+			CommonSimpleParams: CommonSimpleParams{
+				Username:      "cc",
+				Password:      NewPassword("c@c*1"),
+				ConnectString: "192.168.1.1/cc",
+			},
 		},
 		PoolParams: PoolParams{
 			MinSessions:      DefaultPoolMinSessions,
 			MaxSessions:      DefaultPoolMaxSessions,
-			SessionIncrement: DefaultPoolIncrement,
 			MaxLifeTime:      DefaultMaxLifeTime,
+			PingInterval:     3142 * time.Millisecond,
+			SessionIncrement: DefaultPoolIncrement,
+			SessionTimeout:   DefaultSessionTimeout,
+			WaitTimeout:      DefaultWaitTimeout,
+		},
+	}
+	wantDefault := ConnectionParams{
+		// StandaloneConnection: DefaultStandaloneConnection,
+		CommonParams: CommonParams{
+			CommonSimpleParams: CommonSimpleParams{
+				Username:      "user",
+				Password:      NewPassword("pass"),
+				ConnectString: "sid",
+			},
+		},
+		ConnParams: ConnParams{
+			ConnClass: DefaultConnectionClass,
+		},
+
+		PoolParams: PoolParams{
+			MinSessions:      DefaultPoolMinSessions,
+			MaxSessions:      DefaultPoolMaxSessions,
+			MaxLifeTime:      DefaultMaxLifeTime,
+			SessionIncrement: DefaultPoolIncrement,
 			SessionTimeout:   DefaultSessionTimeout,
 			WaitTimeout:      DefaultWaitTimeout,
 		},
 	}
 	wantDefault.ConnClass = DefaultConnectionClass
-	if wantDefault.StandaloneConnection {
+	if wantDefault.StandaloneConnection.Valid && wantDefault.StandaloneConnection.Bool {
 		wantDefault.ConnClass = ""
 	}
 
@@ -61,8 +70,8 @@ func TestParse(t *testing.T) {
 	wantXO.ConnectString = "localhost/sid"
 
 	wantHeterogeneous := wantDefault
-	wantHeterogeneous.Heterogeneous = true
-	wantHeterogeneous.StandaloneConnection = false
+	wantHeterogeneous.Heterogeneous = Bool(true)
+	wantHeterogeneous.StandaloneConnection = Bool(false)
 	wantHeterogeneous.ConnClass = DefaultConnectionClass
 	//wantHeterogeneous.PoolParams.Username, wantHeterogeneous.PoolParams.Password = "", ""
 	wantHeterogeneous.ConnectString = "localhost/sid"
@@ -131,17 +140,21 @@ func TestParse(t *testing.T) {
 		"simple":   {In: "user/pass@sid", Want: wantDefault},
 		"userpass": {In: "user/pass", Want: wantEmptyConnectString},
 
-		"full": {In: "oracle://user:pass@sid/?poolMinSessions=3&poolMaxSessions=9&poolIncrement=3&connectionClass=TestClassName&standaloneConnection=0&sysoper=1&sysdba=0&poolWaitTimeout=200ms&poolSessionMaxLifetime=4000s&poolSessionTimeout=2000s",
+		"full": {In: "oracle://user:pass@sid/?poolMinSessions=3&poolMaxSessions=9&poolIncrement=3&connectionClass=TestClassName&standaloneConnection=0&adminRole=SYSOPER&poolWaitTimeout=200ms&poolSessionMaxLifetime=4000s&poolSessionTimeout=2000s&pingInterval=4s",
 			Want: ConnectionParams{
+				StandaloneConnection: Bool(false),
 				CommonParams: CommonParams{
-					Username: "user", Password: NewPassword("pass"), ConnectString: "sid",
+					CommonSimpleParams: CommonSimpleParams{
+						Username: "user", Password: NewPassword("pass"), ConnectString: "sid",
+					},
 				},
 				ConnParams: ConnParams{
-					ConnClass: "TestClassName", IsSysOper: true,
+					ConnClass: "TestClassName", AdminRole: SysOPER,
 				},
 				PoolParams: PoolParams{
 					MinSessions: 3, MaxSessions: 9, SessionIncrement: 3,
 					WaitTimeout: 200 * time.Millisecond, MaxLifeTime: 4000 * time.Second, SessionTimeout: 2000 * time.Second,
+					PingInterval: 4 * time.Second,
 				},
 			},
 		},
@@ -157,9 +170,11 @@ func TestParse(t *testing.T) {
 		"ipv6": {
 			In: "oracle://[::1]:12345/dbname",
 			Want: ConnectionParams{
-				StandaloneConnection: DefaultStandaloneConnection,
+				// StandaloneConnection: DefaultStandaloneConnection,
 				CommonParams: CommonParams{
-					ConnectString: "[::1]:12345/dbname",
+					CommonSimpleParams: CommonSimpleParams{
+						ConnectString: "[::1]:12345/dbname",
+					},
 				},
 				ConnParams: ConnParams{
 					ConnClass: cc,
@@ -167,7 +182,7 @@ func TestParse(t *testing.T) {
 				PoolParams: PoolParams{
 					MinSessions: 1, MaxSessions: 1000, SessionIncrement: 1,
 					WaitTimeout: 30 * time.Second, MaxLifeTime: 1 * time.Hour, SessionTimeout: 5 * time.Minute,
-					ExternalAuth: true,
+					// ExternalAuth: Bool(true),
 				},
 			},
 		},
@@ -187,14 +202,17 @@ func TestParse(t *testing.T) {
 			libDir="/Users/cjones/instantclient_19_3"`,
 			Want: wantLibDir},
 
-		"onInit": {In: "oracle://user:pass@sid/?poolMinSessions=3&poolMaxSessions=9&poolIncrement=3&connectionClass=TestClassName&standaloneConnection=0&sysoper=1&sysdba=0&poolWaitTimeout=200ms&poolSessionMaxLifetime=4000s&poolSessionTimeout=2000s&onInit=a&onInit=b",
+		"onInit": {In: "oracle://user:pass@sid/?poolMinSessions=3&poolMaxSessions=9&poolIncrement=3&connectionClass=TestClassName&standaloneConnection=0&adminRole=SYSOPER&poolWaitTimeout=200ms&poolSessionMaxLifetime=4000s&poolSessionTimeout=2000s&onInit=a&onInit=b",
 			Want: ConnectionParams{
+				StandaloneConnection: Bool(false),
 				CommonParams: CommonParams{
-					Username: "user", Password: NewPassword("pass"), ConnectString: "sid",
+					CommonSimpleParams: CommonSimpleParams{
+						Username: "user", Password: NewPassword("pass"), ConnectString: "sid",
+					},
 					OnInitStmts: []string{"a", "b"},
 				},
 				ConnParams: ConnParams{
-					ConnClass: "TestClassName", IsSysOper: true,
+					ConnClass: "TestClassName", AdminRole: SysOPER,
 				},
 				PoolParams: PoolParams{
 					MinSessions: 3, MaxSessions: 9, SessionIncrement: 3,
@@ -216,21 +234,18 @@ func TestParse(t *testing.T) {
 				t.Errorf("parse of %q\ngot\n\t%#v,\nwanted\n\t%#v\n%s", tCase.In, P, tCase.Want, diff)
 				return
 			}
-			// FIXME(tgulacsi): this breaks logfmt
-			if false {
-				s := setP(P.String(), P.Password.Secret())
-				Q, err := Parse(s)
-				if err != nil {
-					t.Errorf("parseConnString %v", err)
-					return
-				}
-				if diff := cmp.Diff(P, Q, cmpOpts...); diff != "" && P.String() != Q.String() {
-					t.Errorf("params got\n\t%+v,\nwanted\n\t%+v\n%s", P, Q, diff)
-					return
-				}
-				if got := setP(Q.String(), Q.Password.Secret()); s != got {
-					t.Errorf("paramString got %q, wanted %q", got, s)
-				}
+			s := setP(P.String(), P.Password.Secret())
+			Q, err := Parse(s)
+			if err != nil {
+				t.Errorf("parseConnString %v", err)
+				return
+			}
+			if diff := cmp.Diff(P, Q, cmpOpts...); diff != "" && P.String() != Q.String() {
+				t.Errorf("params got\n\t%+v,\nwanted\n\t%+v\n%s", P, Q, diff)
+				return
+			}
+			if got := setP(Q.String(), Q.Password.Secret()); s != got {
+				t.Errorf("paramString got %q, wanted %q", got, s)
 			}
 		})
 	}
@@ -273,6 +288,21 @@ func TestSplitQuoted(t *testing.T) {
 	}
 }
 
+func TestStandalone(t *testing.T) {
+	a, err := Parse(`user=scott password=tiger connectString="dbhost:1521/orclpdb1?connect_timeout=2"`)
+	if err != nil {
+		t.Fatal(err)
+	}
+	a.StandaloneConnection = Bool(false)
+	b, err := Parse(a.StringWithPassword())
+	if err != nil {
+		t.Fatal(err)
+	}
+	if a.StandaloneConnection != b.StandaloneConnection {
+		t.Errorf("standalone is set to %t and parsed as %t", a.StandaloneConnection, b.StandaloneConnection)
+	}
+}
+
 func ExampleAppendLogfmt() {
 	var buf strings.Builder
 	AppendLogfmt(&buf, "user", "scott")
@@ -285,7 +315,7 @@ func ExampleAppendLogfmt() {
 	// connectString="dbhost:1521/orclpdb1?connect_timeout=2"
 }
 
-func ExampleConnectString() {
+func ExampleParse() {
 	var P ConnectionParams
 	P.Username, P.Password = "scott", NewPassword("tiger")
 	P.ConnectString = "dbhost:1521/orclpdb1?connect_timeout=2"
@@ -296,8 +326,5 @@ func ExampleConnectString() {
 	// Output:
 	// user=scott password=tiger connectString="dbhost:1521/orclpdb1?connect_timeout=2"
 	// alterSession="NLS_NUMERIC_CHARACTERS=,." alterSession="NLS_LANGUAGE=FRENCH"
-	// configDir= connectionClass= enableEvents=0 externalAuth=0 heterogeneousPool=0
-	// libDir= newPassword= noTimezoneCheck=0 poolIncrement=0 poolMaxSessions=0 poolMinSessions=0
-	// poolSessionMaxLifetime=0s poolSessionTimeout=42s poolWaitTimeout=0s prelim=0
-	// standaloneConnection=0 sysasm=0 sysdba=0 sysoper=0 timezone=
+	// connectionClass= newPassword= poolIncrement=0 poolMinSessions=0 poolSessionTimeout=42s
 }
